# 预测回调


``` python
! [ -e /content ] && pip install -Uqq fastai  # 在Colab上升级fastai
```

``` python
from __future__ import annotations
from fastai.basics import *
```

``` python
from nbdev.showdoc import *
from fastai.test_utils import *
```

> 各种回调用于自定义获取预测的行为

## MCDropoutCallback

> 在推理过程中启用 dropout，允许您多次调用 Learner.get_preds 来通过
> [Monte Carlo Dropout](https://arxiv.org/pdf/1506.02142.pdf)
> 近似模型的不确定性。

``` python
class MCDropoutCallback(Callback):
    def before_validate(self):
        for m in [m for m in flatten_model(self.model) if 'dropout' in m.__class__.__name__.lower()]:
            m.train()
    
    def after_validate(self):
        for m in [m for m in flatten_model(self.model) if 'dropout' in m.__class__.__name__.lower()]:
            m.eval()
```

``` python
learn = synth_learner()

# 调用 get_preds 10 次，然后将预测结果堆叠起来，生成一个形状为 [样本数, 批次大小, ...] 的张量。
dist_preds = []
for i in range(10):
    preds, targs = learn.get_preds(cbs=[MCDropoutCallback()])
    dist_preds += [preds]

torch.stack(dist_preds).shape
```

    torch.Size([10, 32, 1])

## 导出 -

``` python
from nbdev import nbdev_export
nbdev_export()
```

    Converted 00_torch_core.ipynb.
    Converted 01_layers.ipynb.
    Converted 02_data.load.ipynb.
    Converted 03_data.core.ipynb.
    Converted 04_data.external.ipynb.
    Converted 05_data.transforms.ipynb.
    Converted 06_data.block.ipynb.
    Converted 07_vision.core.ipynb.
    Converted 08_vision.data.ipynb.
    Converted 09_vision.augment.ipynb.
    Converted 09b_vision.utils.ipynb.
    Converted 09c_vision.widgets.ipynb.
    Converted 10_tutorial.pets.ipynb.
    Converted 11_vision.models.xresnet.ipynb.
    Converted 12_optimizer.ipynb.
    Converted 13_callback.core.ipynb.
    Converted 13a_learner.ipynb.
    Converted 13b_metrics.ipynb.
    Converted 14_callback.schedule.ipynb.
    Converted 14a_callback.data.ipynb.
    Converted 15_callback.hook.ipynb.
    Converted 15a_vision.models.unet.ipynb.
    Converted 16_callback.progress.ipynb.
    Converted 17_callback.tracker.ipynb.
    Converted 18_callback.fp16.ipynb.
    Converted 18a_callback.training.ipynb.
    Converted 19_callback.mixup.ipynb.
    Converted 20_interpret.ipynb.
    Converted 20a_distributed.ipynb.
    Converted 21_vision.learner.ipynb.
    Converted 22_tutorial.imagenette.ipynb.
    Converted 23_tutorial.vision.ipynb.
    Converted 24_tutorial.siamese.ipynb.
    Converted 24_vision.gan.ipynb.
    Converted 30_text.core.ipynb.
    Converted 31_text.data.ipynb.
    Converted 32_text.models.awdlstm.ipynb.
    Converted 33_text.models.core.ipynb.
    Converted 34_callback.rnn.ipynb.
    Converted 35_tutorial.wikitext.ipynb.
    Converted 36_text.models.qrnn.ipynb.
    Converted 37_text.learner.ipynb.
    Converted 38_tutorial.text.ipynb.
    Converted 39_tutorial.transformers.ipynb.
    Converted 40_tabular.core.ipynb.
    Converted 41_tabular.data.ipynb.
    Converted 42_tabular.model.ipynb.
    Converted 43_tabular.learner.ipynb.
    Converted 44_tutorial.tabular.ipynb.
    Converted 45_collab.ipynb.
    Converted 46_tutorial.collab.ipynb.
    Converted 50_tutorial.datablock.ipynb.
    Converted 60_medical.imaging.ipynb.
    Converted 61_tutorial.medical_imaging.ipynb.
    Converted 65_medical.text.ipynb.
    Converted 70_callback.wandb.ipynb.
    Converted 71_callback.tensorboard.ipynb.
    Converted 72_callback.neptune.ipynb.
    Converted 73_callback.captum.ipynb.
    Converted 74_callback.cutmix.ipynb.
    Converted 97_test_utils.ipynb.
    Converted 99_pytorch_doc.ipynb.
    Converted index.ipynb.
    Converted tutorial.ipynb.

``` python

```
